/* part_hash.c -- hash file database access to parts */
#include "rec_io.h"
#include "part.h"
#define HPART struct hash_part
HPART
	{
	char hcode;
	PART part;
	};
static HPART hpart = {0};
static HPART kpart = {0};
static rec_fd part_fd = 0;

/* db_cmp_part - compare function for database access */
static int db_cmp_part(p1, p2)
	HPART *p1, *p2;
	{
	return (strcmp(p1->part.part_no, p2->part.part_no));
	}																					                                                                         /*
.PE
.PS 11
/* db_hash_part - hash the part_no field */
static long db_hash_part(p1)
	HPART *p1;
	{
	char *p;
	long sum = 0;

	for (p = p1->part.part_no; *p != '\0'; ++p)
		sum += *p;
	return (IMOD(sum, rec_nrecs(part_fd)));
	}																					                                                                         /*
.PE
.PS 7
/* db_open_part - open the parts database */
void db_open_part()
	{
	part_fd = rec_open("part.dat", O_RDWR, sizeof(HPART));
	if (part_fd < 0)
		error("can't open ", "part.dat");
	}																					                                                                         /*
.PE
.PS 6
/* db_close_part - close the parts database */
void db_close_part()
	{
	if (rec_close(part_fd) < 0)
		error("can't close ", "part.dat");
	}																					                                                                         /*
.PE
.PS 21
/* db_add_part - add a part to the database */
bool db_add_part(partp)
	PART *partp;
	{
	long i;

	STRUCTASST(hpart.part, *partp);
	hpart.hcode = REC_OCCUP;
	i = rec_havail(part_fd, &hpart, &kpart, db_hash_part);
	if (i == REC_FULL)
		{
		remark("Part storage is full", "");
		return (NO);
		}
	else if (i == REC_ERROR || !rec_put(part_fd, &hpart, i))
		{
		remark("I/O error", "");
		return (NO);
		}
	return (YES);
	}																					                                                                         /*
.PE
.PS 11
/* db_chg_part - change a part record on database */
bool db_chg_part(partp)
	PART *partp;
	{
	long this_rec;
	
	STRUCTASST(kpart.part, *partp);
	this_rec = rec_hfind(part_fd, &kpart, &hpart, db_cmp_part, db_hash_part);
	kpart.hcode = REC_OCCUP;
	return (rec_put(part_fd, &kpart, this_rec));
	}																					                                                                         /*
.PE
.PS 11
/* db_del_part - delete a part record on database */
bool db_del_part(partp)
	PART *partp;
	{
	long this_rec;
	
	STRUCTASST(kpart.part, *partp);
	this_rec = rec_hfind(part_fd, &kpart, &hpart, db_cmp_part, db_hash_part);
	kpart.hcode = REC_DELET;
	return (rec_put(part_fd, &kpart, this_rec));
	}																					                                                                         /*
.PE
.PS 21
/* db_find_part - find a part on the database */
bool db_find_part(partp)
	PART *partp;
	{
	long this_rec;

	STRUCTASST(kpart.part, *partp);
	this_rec = rec_hfind(part_fd, &kpart, &hpart, db_cmp_part, db_hash_part);
	if (this_rec >= 0)
		{
		STRUCTASST(*partp, hpart.part);
		return (YES);
		}
	else if (this_rec == REC_ERROR)
		{
		remark("I/O error", "");
		return (NO);
		}
	else
		return (NO);
	}
